import React from 'react';
import { VDomModel, VDomRenderer } from '@jupyterlab/apputils';
import '../../../../style/diagnostics_listing.css';
import { diagnosticSeverityNames } from '../../../lsp';
import { message_without_code } from './diagnostics';
export class DiagnosticsDatabase extends Map {
    get all() {
        return [].concat.apply([], this.values());
    }
}
function focus_on(node) {
    if (!node) {
        return;
    }
    node.scrollIntoView();
    node.focus();
}
function DocumentLocator(props) {
    let { document, editor } = props;
    let ancestry = document.ancestry;
    let target_cell = null;
    let breadcrumbs = ancestry.map(document => {
        if (!document.parent) {
            let path = document.path;
            if (!editor.has_lsp_supported_file &&
                path.endsWith(document.file_extension)) {
                path = path.slice(0, -document.file_extension.length - 1);
            }
            return React.createElement("span", { key: document.uri }, path);
        }
        if (!document.virtual_lines.size) {
            return React.createElement("span", { key: document.uri }, "Empty document");
        }
        try {
            if (editor.has_cells) {
                let first_line = document.virtual_lines.get(0);
                let last_line = document.virtual_lines.get(document.last_virtual_line - 1);
                let notebook_editor = editor;
                let { cell_id: first_cell, cell } = notebook_editor.find_cell_by_editor(first_line.editor);
                let { cell_id: last_cell } = notebook_editor.find_cell_by_editor(last_line.editor);
                target_cell = cell;
                let cell_locator = first_cell === last_cell
                    ? `cell ${first_cell + 1}`
                    : `cells: ${first_cell + 1}-${last_cell + 1}`;
                return (React.createElement("span", { key: document.uri },
                    document.language,
                    " (",
                    cell_locator,
                    ")"));
            }
        }
        catch (e) {
            console.warn('LSP: could not display document cell location', e);
        }
        return React.createElement("span", { key: document.uri }, document.language);
    });
    return (React.createElement("div", { className: 'lsp-document-locator', onClick: () => focus_on(target_cell ? target_cell.node : null) }, breadcrumbs));
}
const sorters = {
    'Virtual Document': (a, b) => {
        return a.document.id_path.localeCompare(b.document.id_path);
    },
    Message: (a, b) => {
        return a.data.diagnostic.message.localeCompare(b.data.diagnostic.message);
    },
    Source: (a, b) => {
        return a.data.diagnostic.source.localeCompare(b.data.diagnostic.source);
    },
    Code: (a, b) => {
        return (a.data.diagnostic.code + '').localeCompare(b.data.diagnostic.source + '');
    },
    Severity: (a, b) => {
        return a.data.diagnostic.severity > b.data.diagnostic.severity ? 1 : -1;
    },
    Line: (a, b) => {
        return a.data.range.start.line > b.data.range.start.line ? 1 : -1;
    },
    Ch: (a, b) => {
        return a.data.range.start.ch > b.data.range.start.ch ? 1 : -1;
    },
    Cell: (a, b) => {
        return a.cell_nr > b.cell_nr ? 1 : -1;
    }
};
export class DiagnosticsListing extends VDomRenderer {
    constructor(model) {
        super();
        this.sort_key = 'Severity';
        this.sort_direction = 1;
        this.model = model;
    }
    sort(key) {
        if (key === this.sort_key) {
            this.sort_direction = this.sort_direction * -1;
        }
        else {
            this.sort_key = key;
            this.sort_direction = 1;
        }
        this.update();
    }
    render() {
        let diagnostics_db = this.model.diagnostics;
        const editor = this.model.virtual_editor;
        if (!diagnostics_db || typeof editor === 'undefined') {
            return React.createElement("div", null, "No issues detected, great job!");
        }
        let documents_count = diagnostics_db.size;
        let by_document = Array.from(diagnostics_db).map(([virtual_document, diagnostics]) => {
            return diagnostics.map((diagnostic_data, i) => {
                let cell_nr = null;
                if (editor.has_cells) {
                    let notebook_editor = editor;
                    let { cell_id } = notebook_editor.find_cell_by_editor(diagnostic_data.editor);
                    cell_nr = cell_id + 1;
                }
                return {
                    data: diagnostic_data,
                    key: virtual_document.uri + ',' + i,
                    document: virtual_document,
                    cell_nr: cell_nr
                };
            });
        });
        let flattened = [].concat.apply([], by_document);
        let sorter = sorters[this.sort_key];
        let sorted = flattened.sort((a, b) => sorter(a, b) * this.sort_direction);
        let elements = sorted.map(({ data, key, document, cell_nr }) => {
            let diagnostic = data.diagnostic;
            let cm_editor = data.editor;
            let message = message_without_code(diagnostic);
            let severity = diagnostic.severity || 1;
            return (React.createElement("tr", { key: key, onClick: () => {
                    focus_on(cm_editor.getWrapperElement());
                    cm_editor.getDoc().setCursor(data.range.start);
                    cm_editor.focus();
                } },
                documents_count > 1 ? (React.createElement("td", null,
                    React.createElement(DocumentLocator, { document: document, editor: editor }))) : null,
                React.createElement("td", null, message),
                React.createElement("td", null, diagnostic.code),
                React.createElement("td", null, diagnosticSeverityNames[severity]),
                React.createElement("td", null, diagnostic.source),
                editor.has_cells ? React.createElement("td", null, cell_nr) : null,
                React.createElement("td", null, data.range.start.line),
                React.createElement("td", null, data.range.start.ch)));
        });
        let SortableTH = (props) => {
            const is_sort_key = props.name === this.sort_key;
            return (React.createElement("th", { onClick: () => this.sort(props.name), className: is_sort_key
                    ? 'lsp-sorted ' +
                        (this.sort_direction === 1 ? 'lsp-descending' : '')
                    : '' },
                props.name,
                is_sort_key ? React.createElement("span", { className: 'lsp-caret' }) : null));
        };
        return (React.createElement("table", { className: 'lsp-diagnostics-listing' },
            React.createElement("thead", null,
                React.createElement("tr", null,
                    documents_count > 1 ? (React.createElement(SortableTH, { name: 'Virtual Document' })) : null,
                    React.createElement(SortableTH, { name: 'Message' }),
                    React.createElement(SortableTH, { name: 'Code' }),
                    React.createElement(SortableTH, { name: 'Severity' }),
                    React.createElement(SortableTH, { name: 'Source' }),
                    editor.has_cells ? React.createElement(SortableTH, { name: 'Cell' }) : null,
                    React.createElement(SortableTH, { name: 'Line' }),
                    React.createElement(SortableTH, { name: 'Ch' }))),
            React.createElement("tbody", null, elements)));
    }
}
(function (DiagnosticsListing) {
    /**
     * A VDomModel for the LSP of current file editor/notebook.
     */
    class Model extends VDomModel {
        constructor() {
            super();
        }
    }
    DiagnosticsListing.Model = Model;
})(DiagnosticsListing || (DiagnosticsListing = {}));
//# sourceMappingURL=diagnostics_listing.js.map