import { LspWsConnection } from 'lsp-ws-connection';
import { until_ready } from './utils';
export class LSPConnection extends LspWsConnection {
    constructor(options) {
        super(options);
        this.closing_manually = false;
    }
    sendSelectiveChange(changeEvent) {
        this._sendChange([changeEvent]);
    }
    sendFullTextChange(text) {
        this._sendChange([{ text }]);
    }
    isRenameSupported() {
        return !!(this.serverCapabilities && this.serverCapabilities.renameProvider);
    }
    async rename(location, newName) {
        if (!this.isConnected || !this.isRenameSupported()) {
            return;
        }
        return new Promise((resolve, reject) => {
            this.connection
                .sendRequest('textDocument/rename', {
                textDocument: {
                    uri: this.documentInfo.documentUri
                },
                position: {
                    line: location.line,
                    character: location.ch
                },
                newName: newName
            })
                .then((result) => {
                this.emit('renamed', result);
                resolve(true);
            }, error => {
                console.warn(error);
                reject(error);
            });
        });
    }
    connect(socket) {
        super.connect(socket);
        until_ready(() => {
            return this.isConnected;
        }, -1)
            .then(() => {
            this.connection.onClose(() => {
                this.isConnected = false;
                this.emit('close', this.closing_manually);
            });
        })
            .catch(() => {
            console.error('Could not connect onClose signal');
        });
        return this;
    }
    close() {
        try {
            this.closing_manually = true;
            super.close();
        }
        catch (e) {
            this.closing_manually = false;
        }
    }
    _sendChange(changeEvents) {
        if (!this.isConnected || !this.isInitialized) {
            return;
        }
        const textDocumentChange = {
            textDocument: {
                uri: this.documentInfo.documentUri,
                version: this.documentVersion
            },
            contentChanges: changeEvents
        };
        this.connection.sendNotification('textDocument/didChange', textDocumentChange);
        this.documentVersion++;
    }
    async getCompletion(location, token, triggerCharacter, triggerKind) {
        if (!this.isConnected) {
            return;
        }
        if (!(this.serverCapabilities && this.serverCapabilities.completionProvider)) {
            return;
        }
        return new Promise(resolve => {
            this.connection
                .sendRequest('textDocument/completion', {
                textDocument: {
                    uri: this.documentInfo.documentUri
                },
                position: {
                    line: location.line,
                    character: location.ch
                },
                context: {
                    triggerKind: triggerKind,
                    triggerCharacter
                }
            })
                .then((params) => {
                if (params) {
                    params = 'items' in params ? params.items : params;
                }
                resolve(params);
            });
        });
    }
}
//# sourceMappingURL=connection.js.map